@@ -7,6 +7,9 @@
 #include "BinFinder.h"
 #include "TGLException.h"
 
+#ifndef R_NO_REMAP
+#  define R_NO_REMAP
+#endif
 #include <R.h>
 #include <Rinternals.h>
 
