@@ -10,6 +10,9 @@
 
 #include "rdbutils.h"
 
+#ifndef R_NO_REMAP
+#  define R_NO_REMAP
+#endif
 #include <R.h>
 #include <Rinternals.h>
 
